#!/bin/bash
# To backup a machine using ZFS on the server and rsync to the client
if [ -t 1 ] ; then 
set -xv
fi
#/usr/local/backupsv2/bin/zfs-rsync.sh www-fleming.ch.private.cam.ac.uk / zpool1/www-fleming.ch.private.cam.ac.uk/zfs-rsnap
HN=$1
SRC=$2
DST=$3

TAG=${SRC//\//.}

set -e
#set -xv
LOGDIR=/var/log/chem-zfs-backup-server/
CONFDIR=/etc/chem-zfs-backup-server/zfs-rsync.d

if [ -z "$DST" ] ; then
 echo Use: $0 hostname destination
 exit 1
fi

if [ -f $CONFDIR/${HN}_${SRC//\//.} ] ; then
 # This file can define PRE and POST as scripts which are run
 echo Including settings from $CONFDIR/${HN}_${TAG}
 . $CONFDIR/${HN}_${TAG}
 export SSHPORT
 export SSHUSER
fi

# Prepare the host if required
if [ ! -z "$PRE" ] ; then
 $PRE $HN >>$LOGDIR/${HN}_${TAG}-pre.log 2>&1
fi
#set -xv
# Do the backup
TIME=`date +%s`
LOGFILE=$LOGDIR${HN}_${TAG}.log
echo Making snapshot with zfs snapshot $DST@$TIME 2>&1 >>$LOGFILE
zfs snapshot $DST@$TIME
#DEST=`mount | awk ' $1=="'$DST'" { print $3 } '`
# Maybe we should mount the ZFS
MNTED=`zfs get -H -ovalue mounted $DST`
if [ $MNTED = no ] ; then
 echo Mounting ZFS $DST 2>&1 >>$LOGFILE
 zfs mount $DST
fi
DEST=`zfs get -H -ovalue mountpoint "$DST"`
[ -d $DEST/.zfs ]  || exit 2 # Quit if no .zfs directory

# Check for an sshfs exclude file
if [ -f $CONFDIR/$HN/sshfs ]
then
 SSHFS=$CONFDIR/$HN/sshfs
fi

echo Running rsync >> $LOGFILE
echo rsync -e \"ssh -p ${SSHPORT:-22} \" -a ${SSHUSER:-root}@$HN:$SRC $DEST/ --exclude-from=$CONFDIR/$HN/exclude --exclude-from=$CONFDIR/global_ignore  ${SSHFS:+--exclude-from=$SSHFS} --delete --delete-excluded $RSYNCARGS 2>&1 >>$LOGFILE
set +e
rsync -e "ssh -p ${SSHPORT:-22} " -a ${SSHUSER:-root}@$HN:$SRC $DEST/ --exclude-from=$CONFDIR/$HN/exclude --exclude-from=$CONFDIR/global_ignore  ${SSHFS:+--exclude-from=$SSHFS}  --delete --delete-excluded $RSYNCARGS >>$LOGFILE 2>&1
EXIT=$?
echo Finished rsync at `date` with exit $EXIT >> $LOGFILE
# We do not care about rsync exit code 24 which is 'partial transfer due to vanished source files'
if [ $EXIT -eq 24 ]
then
 EXIT=0
fi
# We should always tidy up.
if [ ! -z "$POST" ] ; then
 $POST $HN >>$LOGDIR/${HN}_${TAG}-post.log 2>&1
fi
exit $EXIT
